/*------------------------------------------------------------------------
	Copyright (c) 2006 - 2011 by Burkhardt Renz. All rights reserved.
	$Id: railway.als 592 2011-03-10 10:22:45Z br $
  ------------------------------------------------------------------------
	Beispiel von Daniel Jackson - kommentiert
  ------------------------------------------------------------------------*/

module lfm/railway

/* Das Bahnnetz besteht aus Segmenten, die in einer
   Richtung befahren werden koennen
*/
sig Seg{
	next: set Seg,
	overlaps: set Seg
}
// next ist irreflexiv
fact{ no iden & next }

// overlaps ist reflexiv
// fact{ all s: Seg | s in s.overlaps }
fact{ Seg<:iden in overlaps }

// overlaps ist symmetrisch
// fact{ all s1, s2: Seg | s1 in s2.overlaps => s2 in s1.overlaps } 
fact{ overlaps = ~overlaps }

/* Zuege fahren im Bahnnetz, dabei befindet sich ein Zug
	 immer auf genau einem Segment.
   Diese Relation haben wir in den TrainState gepackt, damit
   wird die Zuege spaeter fahren lassen koennen.
   Auszerdem haben wir eine Relation occupied, die gerade die
   Segmente enthaelt, die in einem Zustand durch einen Zug
   besetzt ist.
*/
sig Train{}
sig TrainState{
	on: Train -> one Seg,
	occupied: set Seg
}
/* Variable, die angibt, ob und welcher Zug sich auf einem
   Segment befindet

   contains = { state: TrainState, seg: Seg, train: Train |
							seg = train.(state.on) }
*/

/* on und occupied haengen natuerlich zusammen:
*/
fact{
	all x: TrainState |
		x.occupied = {s: Seg | some t: Train | t.(x.on) = s}
}

pred findModel{
	#TrainState > 0
	#Seg > 3
	#Train > 0
	some next
}
run findModel for 5


/* Die Segmente haben ein Gate an ihrem Ende, welche der
   Segmente geschlossen sind, wird in GateState festgehalten
*/
sig GateState{
	closed: set Seg
}

/* Ein Zustand ist sicher, wenn sich auf ueberlappenden
   Segmenten hoechstens ein Zug befindet
*/
pred Safe[ x: TrainState ]{
	all s: Seg | lone s.overlaps.~(x.on) 
}

/* Jetzt lassen wir die Zuege fahren
   die Menge ts von Zuegen faehrt auf ein folgendes Segment,
   die restlichen Zuege bleiben stehen
*/
pred TrainsMove[ x, x': TrainState, ts: set Train ]{
	all t: ts | t.(x'.on) in t.(x.on).next
	all t: Train - ts | t.(x'.on) = t.(x.on)
}

pred TrainsMoving[ x, x': TrainState, ts: set Train ]{
	#Train = 1
	#TrainState > 1
	#Seg > 1
	TrainsMove[ x, x', ts ]
}
run TrainsMoving for 1 Train, 2 TrainState, 1 GateState, 4 Seg

/* Aber die Zuege duerfen nur fahren, wenn sie nicht auf
   einem Segment stehen, dessen Gate geschlossen ist
*/

pred MayMove[ g: GateState, x: TrainState, ts: set Train ]{
	no ts.(x.on) & g.closed
  -- Segmente auf denen sich bewegende Zuege stehen sind
	-- nicht in g.closed
}

/* Und nun die Sicherheitsrichtlinie:
   (1) alle Vorgaenger eines besetzten Segments haben ihr
       Gate geschlossen.
   (2) fuer zwei ueberlappende Segmente gibt es hoechstens
       ein offenes Gate.
*/
pred GatePolicy[ g: GateState, x: TrainState ]{
	x.occupied.overlaps.~next in g.closed
  all s1, s2: Seg |
		some s1.next.overlaps & s2.next => lone (s1+s2)-g.closed
}

/* funktioniert diese Sicherheitsrichtlinie? */
assert PolicyWorks{
	all x, x': TrainState, g: GateState, ts: set Train |
		{ MayMove[ g, x, ts ]
		  TrainsMove[ x, x', ts ]
      Safe[ x ]
      GatePolicy[ g, x ] } => Safe[ x' ]
}

check PolicyWorks for 2 Train, 1 GateState, 2 TrainState, 4 Seg

