/*------------------------------------------------------------------------
	Copyright (c) 2006 - 2011 by Burkhardt Renz. All rights reserved.
	$Id: telefon.als 592 2011-03-10 10:22:45Z br $
  ------------------------------------------------------------------------
	Example of feature interaction
  ------------------------------------------------------------------------*/

module lfm/telefon

/*
	Example for a feature interaction:
	feature F1 blocking calls, feature F2 forwarding calls
	Alice won't be called from Bob, so she puts Bob on her blocklist.
	Mark is not at home, he forwards his calls to Alice.
	Bob calls Mark and reaches Alice although his on Alice's blocklist!
*/

sig Person{
	// persons I dont want to be called
	block: set Person,	
	// person I forward calls to 
	forward: lone Person,
	// persons I can directly reach
	direct: set Person
}{
	// forward only to other person
	this != forward
	// forward only if not blocked
	all q: Person | this in q.@block => q !in forward 
	// block only others
	this !in block
	// direct reachable
	all q: Person | 
		(this != q and this !in q.@block) <=> q in direct 
}
/* 
	Remark on the language: if the facts are defined as second part of the
  definition of a signature, this implicitly means all this: Person.
  Each field f therefore means this.f. To prevent this expansion on 
  uses @.
  In the example above: forward is this.forward, and
  q.@block is the block set of q
*/


pred isReachable[ caller: Person, wanted: Person ]{
	// wanted is directly or indirectly reachable from caller
	wanted in caller.direct or wanted in (caller.direct).^forward
}

pred findModel{
	#Person > 1
}
run findModel

// Claim: A person can not reach herself?
assert NoSelfCall{
	all p: Person | !isReachable[p, p]
}
check NoSelfCall

// Claim: Can't be reached by blocked person?
assert BlockHolds{
	// p blocked q so p is not reachable from q
	all p, q: Person | q in p.block => !isReachable[q, p]
}
check BlockHolds
